package com.insanityengine.ghia.apps;

import java.applet.*;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import java.util.*;

import com.insanityengine.ghia.m3.*;
import com.insanityengine.ghia.util.*;
import com.insanityengine.ghia.pixels.*;
import com.insanityengine.ghia.libograf.*;

/**
 *
 * <P>
 * This play pretty is that kinda pipe screen saver thing you like.
 * It needs to be used with a renderer which implements a real pixel
 * buffer type interface...
 * </P>
 *
 * <IMG SRC="http://ghia.insanityengine.com/images/old_snaps/Piping.gif"/>
 *
 * @author BrianHammond
 *
 */

public class Piping extends AppletBase {

	/////////////////////////////////////////////////////////////

	public final static void main( String argv[] ) {
		Piping lite = new Piping();
		lite.frame( 600, 600 );
	}
	
	/////////////////////////////////////////////////////////////
	
	public Piping() {
		initCirclePoints();
		reset();
	}
	
	/////////////////////////////////////////////////////////////
	
	public void initImpl( int w, int h ) {
		gl.getRenderer().setShadingMethod( M3_Constants.NORM_SHADE );
		gl.getRenderer().clear();
	}

	public final void drawImpl() {
		
		gl.identity();
		gl.rotate( rotation );
		gl.scale( scale );

		pipe.draw( gl );

		if ( paused ) {
			reset();
			paused = false;
		}
	}

	public final void reset() {
		if ( null != gl ) gl.getRenderer().clear();
		scale.set( ( float ) ( 0.01 + 0.05 * Math.random() ) );
		rotation.set(
			( float ) ( Math.PI * 2 * Math.random() )
			,
			( float ) ( Math.PI * 2 * Math.random() )
			,
			( float ) ( Math.PI * 2 * Math.random() )
		);
		pipe.reset();
	}

	////////////////////////////////////////////////////////////

	Piper pipe = new Piper();
	private static Pt2 circlePts [] = null;
	private Pt3 scale = new Pt3();
	private Pt3 rotation = new Pt3();

	////////////////////////////////////////////////////////////
	
	private void initCirclePoints() {
		if ( null == circlePts ) {

			circlePts = new Pt2[ 16 ];
			int i = 0;
			double rad = 1f;
			double inc = Math.PI * 2 / circlePts.length;

			for ( double f = 0 ; f < Math.PI * 2 ; f += inc, ++i ) {
				System.out.println( "" + " f = " + f );

				circlePts[ i ] = new Pt2(
					rad * Math.cos( f )
					,
					rad * Math.sin( f )
				);
			}
		}
	}

	////////////////////////////////////////////////////////////

	/**
	 *
	 * This class is used to draw the pipes
	 *
	 */
	private class Piper {

		/**
		 *
		 * Constructor 
		 *
		 */
		public Piper() {
			int[] sk = { 0x000000FF };
			skin = new ImageSkin( 1, 1, sk );
		}

		public void reset() {
			count = 0;
		}

		/**
		 *
		 * Reset to a random point and direction
		 *
		 */
		public void reset( LiboGrafInterface gl ) {
			mat.identity();
			
			// translate to some point
			mat.translate( 
				new Pt3(
					( float ) ( Math.random() - Math.random() ),
					( float ) ( Math.random() - Math.random() ),
					( float ) ( Math.random() - Math.random() )
				)
			);
			
			// random color
			skin.getPixels()[ 0 ] = ( int ) ( 0x00FFFFFF * Math.random() );

			// for some period of time
			count = Stdlib.irand() % 20 + 10;

			// pick a random angle
			newAngle( gl );
		}

		/**
		 *
		 * Turn 90 degrees in a random dimension
		 *
		 */
		private void newAngle( LiboGrafInterface gl ) {

			// 80% chance of a real change...
			if ( 0.80f > Math.random() ) {
			
				float f = ( 44.0f / 7.0f * 0.25f ) * ( Stdlib.rand() % 4 );
				
				float currx, curry, currz;
				currx = curry = currz = 0;

				int next = Stdlib.irand() % 3;
				if ( -1 != last && last != next ) {
					com.insanityengine.ghia.GeomLoaded.GeomLoadedSphere2.staticDraw( gl );
				}

				switch( next ) {
					case 0: currx = f; break;
					case 1: curry = f; break;
					case 2: currz = f; break;
				}

				last = next;

				// point some 90 degree angle
				mat.rotate( new Pt3( currx, curry, currz ) );
				
				if ( 0 == Stdlib.irand() % 2 ) inc *= -1;
			}
		}

		/**
		 *
		 * Draw the pipe and move on
		 *
		 * @param gl context to draw in
		 *
		 */
		public void draw( LiboGrafInterface gl ) {

			inc = ( float ) ( 10 + Math.random() * 30 );
			
			if ( 0 >= count ) reset( gl );

			gl.multiply( mat );

			gl.getRenderer().setSkin( skin );

			drawPipe( gl );
	
			// move on...
			mat.translate( new Pt3( 0, 0, inc ) );
			
			// change direction
			newAngle( gl );

			count--;
		}

		/**
		 *
		 * Draw the pipe using the LiboGrafInterface
		 *
		 * @param gl to draw to
		 *
		 */
		private void drawPipe( LiboGrafInterface gl ) {
			
			for ( int i = 0, j ; i < circlePts.length ; i++ ) {
				j = ( circlePts.length == i ) ? ( i + 1 ) : ( 0 );
				
				gl.startPolygon();
					ptIt( gl, circlePts[ i ].getX(), circlePts[ i ].getY(), 0 );
					ptIt( gl, circlePts[ j ].getX(), circlePts[ j ].getY(), 0 );
					ptIt( gl, circlePts[ j ].getX(), circlePts[ j ].getY(), inc );
					ptIt( gl, circlePts[ i ].getX(), circlePts[ i ].getY(), inc );
				gl.stopPolygon();
			}
		}

		/**
		 *
		 * Add a point to the LiboGrafInterface
		 *
		 * @param gl to addPoint to
		 * @param x coordinate
		 * @param y coordinate
		 * @param z coordinate
		 *
		 */
		private void ptIt( LiboGrafInterface gl, double x, double y, double z ) {
			pt.set( ( float ) x, ( float ) y, ( float ) z );
			gl.addPoint( pt );
		}

		//////////////////////////////////////////////////////

		private Pt3 pt = new Pt3();
	
		private int last = -1;	
		private int count = 0;
		private float inc = 10.0f;

		private ImageSkin skin = null;

		private Matrix mat = new Matrix();
	};
	
};

/**
 *
 * $Log: Piping.java,v $
 * Revision 1.6  2005/03/26 23:42:41  brian
 * fix links
 *
 * Revision 1.5  2005/03/19 17:50:02  brian
 * repackaging
 *
 * Revision 1.4  2005/03/11 23:35:31  brian
 * major refactoring to add in com.insanityengine.ghia.libograf.State bitz
 *
 * Revision 1.3  2004/09/27 00:26:19  brian
 * added image link to dox
 *
 * Revision 1.2  2004/09/22 03:17:15  brian
 * be less lame...
 *
 *
 */
