package com.insanityengine.ghia.events;

import java.awt.event.*;
import javax.swing.event.*;

/**
 *
 *  ActionEventGenerator, is a simplistic event generator pal for you.
 *  ActionListeners clients will need to implement:
 *
 *    	public void actionPerformed( ActionEvent e );
 * 
 *  From java.awt.event.ActionListener interface.  To use this class
 *  subclass it (and blow your one inheritance... :P) or cut-and-paste
 *  this:
 *
 *
 *		ActionEventGenerator actionAl = new ActionEventGenerator( this );
 *		
 *		public void addActionListener( ActionListener l ) {
 *			actionAl.addActionListener( l );
 *		}
 *		
 *		public void removeActionListener( ActionListener l ) {
 *			actionAl.removeActionListener( l );
 *		}
 *	
 *		public void fire( String s ) { actionAl.fire( s ); }
 *
 *
 *  to generate an event do something like this:
 *
 *		actionAl.fire( "some message" );
 *
 *	any listeners will get an ActionEvent with your class as the 
 *	return for ActionEvent.getSource() and your message from
 *  ActionEvent.getActionCommand(), so make shore yo sh!t is tite!
 *
 *	Special thanks to:  JFileChooser.java, which I ripped off long ago...
 *
 */
public class ActionEventGenerator {

	/**
	 * public ActionEventGenerator() {
	 * @param x
	 * @return x
	 */
	public ActionEventGenerator() {
		setOwner( this );
	}

	/**
	 * public ActionEventGenerator( Object owner ) {
	 * @param x
	 * @return x
	 */
	public ActionEventGenerator( Object owner ) {
		setOwner( owner );
	}

    /**
     * public void addActionListener( ActionListener l ) {
     * @param x
     * @return x
     */
    public void addActionListener( ActionListener l ) {
		listenerList.add( ActionListener.class, l );
    }

    /**
     * public void removeActionListener( ActionListener l ) {
     * @param x
     * @return x
     */
    public void removeActionListener( ActionListener l ) {
        listenerList.remove( ActionListener.class, l );
    }

    /**
     * protected void fireActionPerformed( String command ) {
     * @param x
     * @return x
     */
    public void fire( String command ) {
		fireActionPerformed( command, owner );
	}

    /**
     * protected void fireActionPerformed( String command ) {
     * @param x
     * @return x
     */
    public void fireActionPerformed( String command ) {
		fireActionPerformed( command, owner );
	}

	/**
	 * Object getOwner() { return owner; }
	 * @param x
	 * @return x
	 */
	Object getOwner() { return owner; }

	/**
	 * void setOwner( Object newValue ) { owner = newValue; }
	 * @param x
	 * @return x
	 */
	void setOwner( Object newValue ) { owner = newValue; }

	/**
	 * void disown() {
	 * @param x
	 * @return x
	 */
	void disown() {
		setOwner( this );
	}

	///////////////////////////////////////////////////////////////////

	private EventListenerList listenerList = new EventListenerList();
	private Object owner = this;
	
    /**
     * private void fireActionPerformed( String command, Object src ) {
     * @param x
     * @return x
     */
    private void fireActionPerformed( String command, Object src ) {

        // Guaranteed to return a non-null array
        Object[] listeners = listenerList.getListenerList();
        ActionEvent e = null;

        // Process the listeners last to first, notifying
        // those that are interested in this event
        for ( int i = listeners.length - 2 ; i >= 0; i -= 2 ) {
            if ( listeners[ i ] == ActionListener.class ) {
                // Lazily create the event:
                if ( null == e ) {
                    e = new ActionEvent( 
						src, 
						ActionEvent.ACTION_PERFORMED, command 
					);
                }
                ( ( ActionListener ) listeners[ i + 1 ] ).actionPerformed( e );
            }
        }
    }

};

/**
 * 
 * $Log: ActionEventGenerator.java,v $
 * Revision 1.3  2005/03/19 17:50:02  brian
 * repackaging
 *
 * Revision 1.2  2004/09/23 00:46:38  brian
 * fix package name
 *
 * Revision 1.1  2004/09/22 23:36:18  brian
 * to generate action events...
 *
 * 
 */
