package com.insanityengine.ghia.libograf;

import java.util.Stack;

import java.awt.*;

import java.applet.Applet;
import java.awt.Component;
import java.awt.image.ImageObserver;

import com.insanityengine.ghia.m3.*;
import com.insanityengine.ghia.pixels.*;
import com.insanityengine.ghia.renderer.*;

/**
 *
 * <P>
 * Very, very, very loosely based on openGL, very loosely! The largest
 * similarity are the matrix stack a similarity in the polygon handling
 * </P>
 *
 * @author BrianHammond
 *
 * $Header: /usr/local/cvsroot/ghia/src/java/com/insanityengine/ghia/libograf/LiboGraf.java,v 1.11 2005/03/19 17:50:02 brian Exp $
 *
 */

public class LiboGraf extends RendererConsumer implements LiboGrafInterface {

	////////////////////////////////////////////////////////

	/**
	 *
	 * Constructor
	 *
	 */
	public LiboGraf() {
		init( 0, 0, null );
	}

	/**
	 *
	 * Constructor
	 *
	 * @param width of the display
	 * @param height of the display
	 * @param component to use for graphics (yuck!)
	 *
	 */
	public LiboGraf( int width, int height, java.awt.Component component ) {
		init( width, height, component );
	}

	/**
	 *
	 * Constructor, if m3_renderName parameter is defined for the 
	 * Applet, it will attempt to use it with the createRenderer
	 * method
	 *
	 * @param applet which provides context
	 *
	 */
	public LiboGraf( Applet applet ) {
		init( applet );
	}

	/**
	 * Initialize this LiboGraf
 	 *
	 * @param width of the display
	 * @param height of the display
	 * @param component to use
	 *
	 */
	public final void init( int width, int height, java.awt.Component component ) {
		init();
		super.init( width, height, component );
		Imago.setComponent( component );
		identity();
	}
	
	/**
	 *
	 * Rotate the current state
	 *
	 * @param angle
	 * 
	 * @return a State
	 * 
	 */
	public final State rotate( Pt3 angle ) {
		return curr.rotate( angle );
	}

	/**
	 *
	 * Translate the current state
	 * 
	 * @param displacement
	 * 
	 * @return a State
	 * 
	 */
	public final State translate( Pt3 displacement ) {
		return curr.translate( displacement );
	}
	
	/**
	 *
	 * Scale the current state
	 *
	 * @param ratio
	 * 
	 * @return a State
	 * 
	 */
	public final State scale( Pt3 ratio ) {
		return curr.scale( ratio );
	}
	
	/**
	 *
	 * Load the current state with the identity state
	 * 
	 * @return a State
	 *
	 */
	public final State identity() {
		return curr.identity();
	}

	/**
	 *
	 * Push the current state down on the stack
	 *
	 */
	public void push() {
		if ( mat_idx >= 32 ) return;
		stack[ mat_idx++ ].set( curr );
	}
	
	/**
	 *
	 * Pop the current state off the stack
	 * 
	 * @return a State
	 *
	 */
	public final State pop() {
		if ( mat_idx > 0 ) {
			curr.set( stack[ --mat_idx ] );
		}
		if ( mat_idx < 0 ) mat_idx = 0;
		return curr;
	}

	/**
	 *
	 * Get the current state
	 *
	 * @return the current state 
	 *
	 */
	public final State get() { 
		return curr; 
	}

	/**
	 *
	 * Set the current state
	 *
	 * @param state
	 *
	 * @return the current state 
	 *
	 */
	public State set( State state ) {
		return curr.set( state );
	}


	/** 
	 * 
	 * multiply
	 * 
	 * @param state
	 *
	 * @return the current state 
	 * 
	 */
	public State multiply( State state ) {
		return curr.multiply( state );
	}

	/**
	 *
	 * Use the Imago to skin an image
	 *
	 * @param href to load 
	 *
	 * @return an ImageSkin, or null on fail
	 *
	 */
	public final ImageSkin skinImage( String href ) {
		return Imago.skinImage( href );
	}

	/**
	 *
	 * Start a new polygon
	 *
	 */
	public final void startPolygon() {
		pt_idx = 0;
		unseen = false;
	}
	
	/**
	 *
	 * Set the texture coordinate for the current point
	 *
	 * @param s coordinate in texture space 
	 * @param t coordinate in texture space
	 *
	 */
	public final void textCoord( float s, float t ) {
		pt_storage[ pt_idx ].setS( s );
		pt_storage[ pt_idx ].setT( t );
	}

	/**
	 *
	 * Add a point (to the current polygon)
	 *
	 * @param x coordinate of the point
	 * @param y coordinate of the point
	 * @param z coordinate of the point
	 *
	 */
	public final void addPoint( float x, float y, float z ) {
		if ( !unseen ) {
			tmp_pt.set( x, y, z );
			curr.multiply( tmp_pt, pt_storage[ pt_idx++ ] );
			
			if ( pt_idx == pt_max ) pt_idx = 0;
		}
	}
	
	/**
	 *
	 * Add a point (to the current polygon)
	 *
	 * @param point to add
	 *
	 */
	public final void addPoint( Pt3 point ) {
		curr.multiply( point, pt_storage[ pt_idx++ ] );
		if ( pt_idx == pt_max ) pt_idx = 0;

		if ( pt_idx == 3 ) {
			norm.normal(  
				pt_storage[ 0 ],
				pt_storage[ 1 ],
				pt_storage[ 2 ]
			);
			getRenderer().setNormal( norm );
		}
	}

	/**
	 *	  
	 * Get the normal of the current polygon
	 *	  
	 * @return the normal of the current polygon
	 *	  
	 */
	public final Pt3 getNormal() {
		return norm;
	}
	
	/**
	 *
	 * Finish the current polygon, invokes the drawPolygon method
	 * on the renderer
	 *
	 */
	public final void stopPolygon() {
		if ( false ) {

			// this doesn't seem to work...
			if ( norm.z >= 0 ) {
				getRenderer().drawPolygon( pt_idx, pt_storage );
			}
			
		} else {
			getRenderer().drawPolygon( pt_idx, pt_storage );
		}
		
		pt_idx = 0;
		norm.set( 0 );
	}

	/**
	 *
	 * Get the maximum number of points a polygon can have
	 *
	 * @return the maximum number of points a polygon can have
	 *
	 */
	public final int getPolygonPointMax() {
		return pt_max;
	}
		
	/**
	 *
	 * Get the size of the matrix stack
	 *
	 * @return the size of the matrix stack
	 *
	 */
	public final int getStackSize() { 
		return mat_max;
	}

	/**
	 *
	 * Silly conveinence method to draw a buncha polygons
	 *
	 * @param polygon array of array of Pt3 representing the polygons
	 *
	 */
	public final void drawPolygons( Pt3 [][] polygon ) {
		int i, j;
		for ( i = 0 ; i < polygon.length ; i++ ) {
			startPolygon();
			for ( j = 0 ; j < polygon[ i ].length ; j++ ) {
				textCoord( polygon[ i ][ j ].x, polygon[ i ][ j ].y );
				addPoint( polygon[ i ][ j ] );
			}
			stopPolygon();
		}
	}

	/**
	 *
	 *
	 *
	 *
	 */
	public void render() {
		render( this );
	}

	/**
	 *
	 *
	 *
	 *
	 */
	public void render( Graphics graphics ) {
		setGraphics( graphics );
		render();
	}

	/**
	 * 
	 * Try to create a LiboGraf given the name of the class 
	 * It will be initialized with the values passed into the init
	 * method of this LiboGraf instance
	 *
	 * @param nameOfLiboGraf to try to use
	 *
	 * @return instance of the LiboGraf on success, null on fail
	 *
	 */
	public final static LiboGrafInterface getLiboGrafByName( String nameOfLiboGraf ) {

		String cause = "uh...";
		LiboGrafInterface yourLiboGraf = null;

		Class theClass = com.insanityengine.ghia.util.DocReflect.forName( nameOfLiboGraf );
		if ( null == theClass ) {
			cause = "class for name failed for " + nameOfLiboGraf;
		} else {

			String interfaceName = "com.insanityengine.ghia.libograf.LiboGrafInterface";
			Class[] interfaces = theClass.getInterfaces();
			cause = nameOfLiboGraf + " does not implement " + interfaceName;

			try {
				yourLiboGraf = ( LiboGrafInterface ) theClass.newInstance();
			} catch ( Exception e ) {
				e.printStackTrace();
				cause = "didn't you see the stacktrace?";
			}
		}

		if ( null == yourLiboGraf ) {
			System.err.println( 
				"LiboGraf.getLiboGrafByName failed: " + cause 
			);
		} else {
			System.err.println( 
				"LiboGraf.getLiboGrafByName succeeded: " + nameOfLiboGraf 
			);
		}

		return yourLiboGraf;
	}

	/**
	 *
	 * Get an instance of the default LiboGrafInterface implementation
	 *
	 * @return an instance of the default LiboGrafInterface implementation
	 *
	 */
	public final static LiboGrafInterface getDefaultLiboGraf() {
		return new LiboGraf();
	}

	/**
	 *
	 * Create a new LiboGrafInterface using the component
	 *
	 * @param component to use 
	 *
	 */
	public final static LiboGrafInterface createLiboGrafInterface( Component component ) {
		LiboGrafInterface impl = null;
		String rname = null;
		String pname = "m3_libografName";

		if ( component instanceof Applet ) {
			try {
				rname = ( ( Applet ) component ).getParameter( pname );
			} catch ( Exception e ) {
			}
		}

		try {
			if ( null == rname ) rname = System.getProperty( pname );
		} catch ( Exception e ) {
		}

		if ( null == rname ) rname = "com.insanityengine.ghia.libograf.LiboGraf";
		return getLiboGrafByName( rname );
	}
	
	/**
	 *
	 * Create a new LiboGrafInterface 
	 *
	 */
	public final static LiboGrafInterface createLiboGrafInterface() {
		return createLiboGrafInterface( null );
	}

	////////////////////////////////////////////////////////
	
	private static final int pt_max = 8;
	private static final int mat_max = 32;

	private State curr;
	private int mat_idx = 0;
	private State[] stack;

	// space for addPoint familia
	private int pt_idx;
	private Pt3 pt_storage[];

	private boolean unseen = false;

	private Pt3 tmp_pt;
	private Pt3 norm;

	private void init() {

		mat_idx = 0;
		pt_idx = 0;

		unseen = false;

		tmp_pt = new Pt3();
		norm = new Pt3();

		pt_storage = new Pt3[ pt_max ];
		for ( int i = 0 ; i < pt_storage.length ; i++ ) {
			pt_storage[ i ] = new Pt3();
		}

		if ( true ) {
			curr = new Matrix();
			stack = new Matrix[ mat_max ];
			for ( int i = 0 ; i < stack.length ; i ++ ) {
				stack[ i ] = new Matrix();
			}
		} else {
			curr = new Quaternion();
			stack = new Quaternion[ mat_max ];
			for ( int i = 0 ; i < stack.length ; i ++ ) {
				stack[ i ] = new Quaternion();
			}
		}
	}
};

/**
 *
 * $Log: LiboGraf.java,v $
 * Revision 1.11  2005/03/19 17:50:02  brian
 * repackaging
 *
 * Revision 1.10  2005/03/12 04:58:08  brian
 * quatz are broke, but provide mechanism to use 'em...
 *
 * Revision 1.9  2005/03/11 23:35:31  brian
 * major refactoring to add in com.insanityengine.ghia.libograf.State bitz
 *
 * Revision 1.8  2005/03/10 13:31:19  brian
 * introduce com.insanityengine.ghia/libograf/State to pave the way for quaternions based LiboGrafInterface implementation
 *
 * Revision 1.7  2004/09/22 03:03:32  brian
 * added setMatrix and multiply methods
 *
 * Revision 1.6  2004/09/11 19:32:29  brian
 * effects of adding LiboGrafInterface
 *
 * Revision 1.5  2004/09/11 16:23:07  brian
 * added getLiboGrafByName and getDefaultLiboGraf static methods
 *
 * Revision 1.4  2004/09/11 14:42:32  brian
 * cleanup
 *
 * Revision 1.3  2004/09/11 14:37:21  brian
 * added RendererConsumer and stuff
 *
 *
 */
