package com.insanityengine.ghia.libograf;

import java.util.Stack;

import java.awt.*;

import java.applet.Applet;
import java.awt.Component;
import java.awt.image.ImageObserver;

import com.insanityengine.ghia.m3.*;
import com.insanityengine.ghia.pixels.*;
import com.insanityengine.ghia.renderer.*;

/**
 *
 * <P>
 * Very, very, very loosely based on openGL, very loosely! The largest
 * similarity are the matrix stack a similarity in the polygon handling
 * </P>
 *
 * @author BrianHammond
 *
 * $Header: /usr/local/cvsroot/ghia/src/java/com/insanityengine/ghia/libograf/LiboGrafInterface.java,v 1.5 2005/03/19 17:50:02 brian Exp $
 *
 */

public interface LiboGrafInterface extends RendererConsumerInterface {

	/**
	 * Initialize this LiboGraf
 	 *
	 * @param width of the display
	 * @param height of the display
	 * @param component to use
	 *
	 */
	public void init( int width, int height, java.awt.Component component );
	
	/**
	 *
	 * Rotate the current state
	 *
	 * @param angle
	 * 
	 * @return a State
	 * 
	 */
	public State rotate( Pt3 angle );

	/**
	 *
	 * Translate the current state
	 * 
	 * @param displacement
	 * 
	 * @return a State
	 * 
	 */
	public State translate( Pt3 displacement );
	
	/**
	 *
	 * Scale the current state
	 *
	 * @param ratio
	 * 
	 * @return a State
	 * 
	 */
	public State scale( Pt3 ratio );
	
	/**
	 *
	 * Load the current state
	 * 
	 * @return a State
	 *
	 */
	public State identity();

	/**
	 *
	 * Push the current state
	 *
	 */
	public void push();
	
	/**
	 *
	 * Pop the current state off the stack
	 * 
	 * @return a State
	 *
	 */
	public State pop();

	/**
	 *
	 * Get the current state
	 *
	 * @return the current state
	 *
	 */
	public State get();

	/**
	 *
	 * Set the current state
	 *
	 * @param state to use
	 * 
	 * @return a State
	 *
	 */
	public State set( State state );
	
	/**
	 *
	 * Multiply the current state by state
	 *
	 * @param state to multiply
	 * 
	 * @return a State
	 *
	 */
	public State multiply( State state );
	
	/**
	 *
	 * Use the Imago to skin an image
	 *
	 * @param href to load 
	 *
	 * @return an ImageSkin, or null on fail
	 *
	 */
	public ImageSkin skinImage( String href );

	/**
	 *
	 * Start a new polygon
	 *
	 */
	public void startPolygon();
	
	/**
	 *
	 * Set the texture coordinate for the current point
	 *
	 * @param s coordinate in texture space 
	 * @param t coordinate in texture space
	 *
	 */
	public void textCoord( float s, float t );

	/**
	 *
	 * Add a point (to the current polygon)
	 *
	 * @param x coordinate of the point
	 * @param y coordinate of the point
	 * @param z coordinate of the point
	 *
	 */
	public void addPoint( float x, float y, float z );
	
	/**
	 *
	 * Add a point (to the current polygon)
	 *
	 * @param point to add
	 *
	 */
	public void addPoint( Pt3 point );

	/**
	 *	  
	 * Get the normal of the current polygon
	 *	  
	 * @return the normal of the current polygon
	 *	  
	 */
	public Pt3 getNormal();
	
	/**
	 *
	 * Finish the current polygon, invokes the drawPolygon method
	 * on the renderer
	 *
	 */
	public void stopPolygon();

	/**
	 *
	 * Get the maximum number of points a polygon can have
	 *
	 * @return the maximum number of points a polygon can have
	 *
	 */
	public int getPolygonPointMax();
		
	/**
	 *
	 * Get the size of the state stack
	 *
	 * @return the size of the state stack
	 *
	 */
	public int getStackSize();

	/**
	 *
	 * Silly conveinence method to draw a buncha polygons
	 *
	 * @param polygon array of array of Pt3 representing the polygons
	 *
	 */
	public void drawPolygons( Pt3 [][] polygon );

	/**
	 *
	 * Render the scene
	 *
	 */
	public void render();

	/**
	 *
	 * Render the scene
	 *
	 * @param graphics to use (if possible)
	 *
	 */
	public void render( Graphics graphics );
};

/**
 *
 * $Log: LiboGrafInterface.java,v $
 * Revision 1.5  2005/03/19 17:50:02  brian
 * repackaging
 *
 * Revision 1.4  2005/03/11 23:35:31  brian
 * major refactoring to add in com.insanityengine.ghia.libograf.State bitz
 *
 * Revision 1.3  2005/03/10 13:31:19  brian
 * introduce com.insanityengine.ghia/libograf/State to pave the way for quaternions based LiboGrafInterface implementation
 *
 * Revision 1.2  2004/09/22 03:03:32  brian
 * added setMatrix and multiply methods
 *
 * Revision 1.1  2004/09/11 19:22:20  brian
 * open the way for jogl LiboGraf impl
 *
 *
 */
