package com.insanityengine.ghia.m3;

/**
 *
 * <P>
 * Why isn't this called P5? I dunno... it is a 5-dimensional point 
 * three coordinates in the "physical" space and two in texture space.
 * </P>
 *
 * @author BrianHammond
 *
 * $Header: /usr/local/cvsroot/ghia/src/java/com/insanityengine/ghia/m3/Pt3.java,v 1.13 2005/03/25 13:48:17 brian Exp $
 *
 */

public class Pt3 {
	

	public float x, y, z;
	public float s, t;

	/**
	 *
	 * Constructor
	 *
	 */
	public Pt3() { 
	}
	
	/**
	 *
	 * Constructor
	 *
	 */
	public Pt3( float d ) {
		set( d );
	}
	
	/**
	 *
	 * Copy Constructor
	 *
	 */
	public Pt3( Pt3 that ) {
		set( that );
	}

	/**
	 *
	 * Constructor
	 *
	 * @param x coordinate
	 * @param y coordinate
	 * @param z coordinate
	 *
	 */
	public Pt3( float x, float y, float z ) {
		set( x, y, z );
	}

	/**
	 *
	 * Set (x,y,z) to d
	 *
	 * @param d value to set to
	 *
	 * @return this
	 *
	 */
	public final Pt3 set( float d ) {
		return set( d, d, d );
	}

	/**
	 *
	 * Set the point to (x,y,z)
	 *
	 * @param x coordinate
	 * @param y coordinate
	 * @param z coordinate
	 *
	 * @return this
	 *
	 */
	public final Pt3 set( float x, float y, float z ) {
		this.x = x; 
		this.y = y; 
		this.z = z;
		return this;
	}

	/**	
	 *
	 * Copy that point
	 *
	 * @param that
	 *
	 * @return this
	 *
	 */
	public final Pt3 set( Pt3 that ) {
		set( that.x, that.y, that.z );
		s = that.s;
		t = that.t;
		return this;
	}

	/**	
	 *
	 * Swap with that point
	 *
	 * @param that point
	 *
	 * @return this
	 *
	 */
	public final Pt3 swap( Pt3 that ) {
		float a = x;
		float b = y;
		float c = z;
		this.x = that.x;
		this.y = that.y;
		this.z = that.z;
		that.x = a; 
		that.y = b;
		that.z = c;
		a = this.s;
		b = this.t;
		this.s = that.s;
		this.t = that.t;
		that.s = a;
		that.t = b;
		return this;
	}

	/**	
	 *
	 * Calculate the dot product with that point
	 *
	 * @param that point to use
	 *
	 * @return dot produce
	 *
	 */
	public final float dot( Pt3 that ) {
		return (
			this.x * that.x + 
			this.y * that.y + 
			this.z * that.z 
		);
	}

	/**	
	 *
	 * Calculate the distance squared to that point
	 *
	 * @param that point to use
	 *
	 * @return squared distance to that point
	 *
	 */
	public final float distance2( Pt3 that ) {
		return (
			+ ( that.x - this.x ) * ( that.x - this.x )
			+ ( that.y - this.y ) * ( that.y - this.y )
			+ ( that.z - this.z ) * ( that.z - this.z )
		);
	}

	/** 
	 * 
	 * add
	 * 
	 * @param d
	 * 
	 * @return a Pt3
	 * 
	 */
	public final Pt3 add( float d ) {
		return add( d, d, d );
	}

	/** 
	 * 
	 * add
	 * 
	 * @param x
	 * @param y
	 * @param z
	 * 
	 * @return a Pt3
	 * 
	 */
	public final Pt3 add( float x, float y, float z ) {
		this.x += x;
		this.y += y;
		this.z += z;
		return this;
	}

	/**	
	 *
	 * Add that point from this point
	 *
	 * @param that point to use
	 *
	 * @return this
	 *
	 */
	public final Pt3 add( Pt3 that ) {
		return add( that.x, that.y, that.z );
	}
	/**	
	 *
	 * Subtract that point from this point
	 *
	 * @param that point to use
	 *
	 * @return this
	 *
	 */
	public final Pt3 subtract( Pt3 that ) {
		this.x -= that.x;
		this.y -= that.y;
		this.z -= that.z;
		return this;
	}
	
	/**	
	 *
	 * Multiply this point by a scalar
	 *
	 * @param scalar to multiply by
	 *
	 * @return this
	 *
	 */
	public final Pt3 multiply( float scalar ) {
		x *= scalar;
		y *= scalar;
		z *= scalar;
		return this;
	}
	
	/**	
	 *
	 * Divide this point by a scalar
	 *
	 * @param scalar to divide by
	 *
	 * @return this
	 *
	 */
	public final Pt3 divide( float scalar ) {
		if ( 0 == scalar ) {
			x /= scalar;
			y /= scalar;
			z /= scalar;
		}
		return this;
	}

	/**	
	 *
	 * Normalize the length of the the point (as a vector) to
	 * a unit vector
	 *
	 * @return this
	 *
	 */
	public final Pt3 normalize() {
		float d = ( float ) Math.sqrt( x * x + y * y + z * z );
		if ( 0 != d ) {
			x /= d;
			y /= d;
			z /= d;
		}
		return this;
	}

	/**	
	 *
	 * Given three points, calculate the normal vector into
	 * this point
	 *
	 * @param pt1 to use
	 * @param pt2 to use
	 * @param pt3 to use
	 *
	 * @return this
	 *
	 */
	public final Pt3 normal( Pt3 pt1, Pt3 pt2, Pt3 pt3 ) {
		float ax = pt1.x - pt2.x;
		float ay = pt1.y - pt2.y;
		float az = pt1.z - pt2.z;

		float bx = pt3.x - pt2.x;
		float by = pt3.y - pt2.y;
		float bz = pt3.z - pt2.z;

		x = ( ay * bz - az * by );
		y = ( az * bx - ax * bz );
		z = ( ax * by - ay * bx );
		
		ax = ( float ) Math.sqrt( x * x + y * y + z * z );
		if ( 0 != ax ) {
			x /= ax;
			y /= ax;
			z /= ax;
		}
		return this;
	}

	/**	
	 *
	 * Subtract pt2 from pt1 and store the result in this point
	 *
	 * @param pt1 to use
	 * @param pt2 to use
	 *
	 * @return this
	 *
	 */
	public final Pt3 minus( Pt3 pt1, Pt3 pt2 ) {
		x = pt1.x - pt2.x;
		y = pt1.y - pt2.y;
	 	z = pt1.z - pt2.z;
		return this;
	}
	
	/**	
	 *
	 * Add pt1 to pt2 and store the result in this point
	 *
	 * @param pt1 to use
	 * @param pt2 to use
	 *
	 * @return this
	 *
	 */
	public final Pt3 plus( Pt3 pt1, Pt3 pt2 ) {
		x = pt1.x + pt2.x;
		y = pt1.y + pt2.y;
	 	z = pt1.z + pt2.z;
		return this;
	}
	
	/**	
	 *
	 * Calculate the cross product and store the result in this point
	 *
	 * @param pt1 to use
	 * @param pt2 to use
	 *
	 * @return this
	 *
	 */
	public final Pt3 cross( Pt3 pt1, Pt3 pt2 ) {
		x = ( pt1.y * pt2.z - pt1.z * pt2.y );
		y = ( pt1.z * pt2.x - pt1.x * pt2.z );
	 	z = ( pt1.x * pt2.y - pt1.y * pt2.x );
		return this;
	}

	/**	
	 *
	 * Multiply the point by the matrix and store the result in this point
	 *
	 * @param matrix to multiply
	 * @param point to multiply
	 *
	 * @return this
	 *
	 */
	public final Pt3 multiply( Mat4 matrix, Pt3 point ) {
		x = (
			point.x * matrix.mat[ 0 ][ 0 ] +
			point.y * matrix.mat[ 0 ][ 1 ] +
			point.z * matrix.mat[ 0 ][ 2 ] +
			      1 * matrix.mat[ 0 ][ 3 ] 
		);

		y = (
			point.x * matrix.mat[ 1 ][ 0 ] +
			point.y * matrix.mat[ 1 ][ 1 ] +
			point.z * matrix.mat[ 1 ][ 2 ] +
			      1 * matrix.mat[ 1 ][ 3 ] 
		);

		z = (
			point.x * matrix.mat[ 2 ][ 0 ] +
			point.y * matrix.mat[ 2 ][ 1 ] +
			point.z * matrix.mat[ 2 ][ 2 ] +
			      1 * matrix.mat[ 2 ][ 3 ] 
		);
		return this;
	}

	/**	
	 *
	 * Convert to string representation
	 *
	 * @return string representation
	 *
	 */
	public String toString() {
		return (
			"pt3" + "(" + x + " , " + y + " , " + z + " )"
		);
	}

	/**
	 *
	 * Get the value of x
	 *
	 * @return the value of x
	 *
	 */
	public final float getX() {
		return x;
	}

	/**
	 *
	 * Set the value of x
	 *
	 * @param newValue of x
	 *
	 */
	public final void setX( float newValue ) {
		x = newValue;
	}

	/**
	 *
	 * Get the value of y
	 *
	 * @return the value of y
	 *
	 */
	public final float getY() {
		return y;
	}

	/**
	 *
	 * Set the value of y
	 *
	 * @param newValue of y
	 *
	 */
	public final void setY( float newValue ) {
		y = newValue;
	}

	/**
	 *
	 * Get the value of z
	 *
	 * @return the value of z
	 *
	 */
	public final float getZ() {
		return z;
	}

	/**
	 *
	 * Set the value of z
	 *
	 * @param newValue of z
	 *
	 */
	public final void setZ( float newValue ) {
		z = newValue;
	}

	/**
	 *
	 * Get the value of s
	 *
	 * @return the value of s
	 *
	 */
	public final float getS() {
		return s;
	}

	/**
	 *
	 * Set the value of s
	 *
	 * @param newValue of s
	 *
	 */
	public final void setS( float newValue ) {
		s = newValue;
	}

	/**
	 *
	 * Get the value of t
	 *
	 * @return the value of t
	 *
	 */
	public final float getT() {
		return t;
	}

	/**
	 *
	 * Set the value of t
	 *
	 * @param newValue of t
	 *
	 */
	public final void setT( float newValue ) {
		t = newValue;
	}

}

/**
 *
 * $Log: Pt3.java,v $
 * Revision 1.13  2005/03/25 13:48:17  brian
 * make toString less verbose...
 *
 * Revision 1.12  2005/03/25 13:08:17  brian
 * make divide less silly
 *
 * Revision 1.11  2005/03/19 17:50:02  brian
 * repackaging
 *
 * Revision 1.10  2005/03/15 05:45:33  brian
 * reduce code dup
 *
 * Revision 1.9  2005/03/12 04:58:47  brian
 * call the methods set instead of copy
 *
 * Revision 1.8  2005/03/11 23:35:31  brian
 * major refactoring to add in com.insanityengine.ghia.libograf.State bitz
 *
 * Revision 1.7  2004/09/01 18:30:00  brian
 * added the add method I should added before but it is added now...
 *
 * Revision 1.6  2004/09/01 01:10:42  brian
 * fix class level javadoc placement
 *
 * Revision 1.5  2004/09/01 00:11:06  brian
 * author, log and header stuff
 *
 *
 */
