package com.insanityengine.ghia.pixelops;

import com.insanityengine.ghia.pixels.PixelBufferReadInterface;

/**
 *
 * <P>
 * Various "post-production", usually 2d effects can be applied to (some) 
 * implementations of the PixelBufferReadInterface
 * </P>
 *
 * @author BrianHammond
 *
 * $Header: /usr/local/cvsroot/ghia/src/java/com/insanityengine/ghia/pixelops/BufferFx.java,v 1.4 2005/03/19 17:50:02 brian Exp $
 *
 */

public class BufferFx {

	/**
	 *
	 * Constructor
	 *
	 */
	public BufferFx() {
	}

	/**
	 *
	 * Constructor
	 *
	 * @param object to try to use
	 *
	 */
	public BufferFx( Object object ) {
		tryToSet( object );
	}
	
	/**
	 *
	 * Constructor
	 *
	 * @param pixelBuffer to use
	 *
	 */
	public BufferFx( PixelBufferReadInterface pixelBuffer ) {
		set( pixelBuffer );
	}

	/**
	 *
	 * Set the PixelBufferReadInterface 
	 * 
	 * @param pixelBuffer to use
	 *
	 */
	public void set( PixelBufferReadInterface pixelBuffer ) {
		this.pixelBuffer = pixelBuffer;
		w = pixelBuffer.getWidth();
		h = pixelBuffer.getHeight();
		max = w * h;
		pixels = pixelBuffer.getPixels();
		clearColor = pixelBuffer.getClearColor();
		if ( null == pixels ) w = h = max = 0;  // this should be enuff...
	}

	/**
	 *
	 * Try to set the PixelBufferReadInterface
	 *
	 * @param object to try with
	 *
	 */
	public void tryToSet( Object object ) {
		if ( objectSupportsFx( object ) ) set( ( PixelBufferReadInterface ) object );
	}

	/**
	 *
	 */
	public void shift() {
		if ( null == pixels ) return; //cheese!
		
		int l = pixels[ max - 1 ];

		System.arraycopy( pixels, 0, pixels, 1, max - 1);
		pixels[ 0 ] = l;
	}

	/**
	 *
	 */
	public PixelBufferReadInterface getBuffer() { 
		return pixelBuffer;
	}

	/**
	 *
	 */
	public void or( int n ) {
		if ( null == pixels ) return; //cheese!
		for ( int i = 0 ; i < max ; i++ ) pixels[ i ] |= n;
	}
	
	/**
	 *
	 */
	public void and( int n ) {
		if ( null == pixels ) return; //cheese!
		for ( int i = 0 ; i < max ; i++ ) pixels[ i ] &= n;
	}

	/**
	 *
	 */
	public void shiftLeft( int n ) {
		if ( null == pixels ) return; //cheese!
		for ( int i = 0 ; i < max ; i++ ) pixels[ i ] = pixels[ i ] << n;
	}
	
	/**
	 *
	 */
	public void shiftRight( int n ) {
		if ( null == pixels ) return; //cheese!
		for ( int i = 0 ; i < max ; i++ ) pixels[ i ] = pixels[ i ] >> n;
	}
	

	/**
	 *
	 */
	public void fade() {
		fade( 1, 2 );
	}

	/**
	 *
	 */
	public void fade( int m, int d ) {
		if ( null == pixels ) return; //cheese!
		int r, g, b;
		for ( int i = 0 ; i < max ; i++ ) {
			r = ( ( pixels[ i ] >> 16 ) & 0xFF ) * m / d;
			g = ( ( pixels[ i ] >>  8 ) & 0xFF ) * m / d;
			b = ( ( pixels[ i ] >>  0 ) & 0xFF ) * m / d;
			pixels[ i ] = ( 255 << 24 | r << 16 | g << 8 | b );
		}
	}

	/**
	 *
	 */
	public void smooth() {
		if ( null == pixels ) return; //cheese!
		int i, r, g, b;

		for ( i = 0 ; i < w + 1 ; i++ ) pixels[ i ] = clearColor;
		for ( i = w + 1 ; i < max ; i++ ) {
			r = ( 
				( ( pixels[ i - w - 1 ] >> 16 ) & 0xFF ) +
				( ( pixels[ i - w - 0 ] >> 16 ) & 0xFF ) +
				( ( pixels[ i - 0 - 1 ] >> 16 ) & 0xFF ) +
				( ( pixels[ i - 0 - 0 ] >> 16 ) & 0xFF ) 
			) >> 2;
			g = ( 
				( ( pixels[ i - w - 1 ] >> 8 ) & 0xFF ) +
				( ( pixels[ i - w - 0 ] >> 8 ) & 0xFF ) +
				( ( pixels[ i - 0 - 1 ] >> 8 ) & 0xFF ) +
				( ( pixels[ i - 0 - 0 ] >> 8 ) & 0xFF ) 
			) >> 2;
			b = ( 
				( pixels[ i - w - 1 ] & 0xFF ) +
				( pixels[ i - w - 0 ] & 0xFF ) +
				( pixels[ i - 0 - 1 ] & 0xFF ) +
				( pixels[ i - 0 - 0 ] & 0xFF ) 
			) >> 2;

			pixels[ i ] = ( 255 << 24 | r << 16 | g << 8 | b );
		}
	}
	
	/**
	 *
	 */
	public void smoothoDoomo() {
		if ( null == pixels ) return; //cheese!
		int i, r, g, b;

		pixels = pixelBuffer.getPixels();

		for ( i = 0 ; i < w + 1 ; i++ ) pixels[ i ] = clearColor;
		for ( i = w + 1 ; i < max ; i++ ) {
			pixels[ i ] = (
				pixels[ i - w - 1 ] +
				pixels[ i - w - 0 ] +
				pixels[ i - 0 - 1 ] +
				pixels[ i - 0 - 0 ] 
				
		 	) >> 2;
		}
	}

	/**
	 *
	 */
	public void eoPerturb() {
		if ( null == pixels ) return; //cheese!
		int i, j, idx;
		for ( i = idx = 0 ; i < h ; ++i ) {
			for ( j = 0 ; j < w ; ++j, ++idx ) {
				pixels[ idx ] += ( i + j ) % 2 * 2 - 1;
			}
		}
	}

	/**
	 *
	 */
	public void hedge() {
		if ( null == pixels ) return; //cheese!
		int i, j, idx;
		for ( i = idx = 0 ; i < h ; ++i ) {
			for ( j = 0 ; j < w ; ++j, ++idx ) {
				pixels[ idx ] += ( i + j ) % 2 * 2 - 1;
			}
		}
	}

	/**
	 *
	 */
	public void topEdge() {
		if ( null == pixels ) return; //cheese!
		for ( int i = 0 ; i < w ; ++i ) pixels[ i + w * 3 ] += 100 * ( 2 * i % 2 - 1 );
	}

	/**
	 *
	 */
	public void statiq() {
		if ( null == pixels ) return; //cheese!
		for ( int i = 0 ; i < max ; ++i ) pixels[ i ] += ( int ) ( 0x00FFFFFF * Math.random() );
	}

	/**
	 *
	 */
	public void avg( int c ) {
		if ( null == pixels ) return; //cheese!
		for ( int i = 0 ; i < max ; ++i ) {
			pixels[ i ] = ( pixels[ i ] | c ) / 2;
		}
	}

	/**
	 *
	 * Test the renderer to see if it support effects
	 *
	 * @param renderer to test
	 * 
	 * @return true if the renderer supports effects
	 * 
	 */
	public final static boolean objectSupportsFx( Object object ) {
		boolean support = false;
		PixelBufferReadInterface reader;

		try {
			
			reader = ( PixelBufferReadInterface ) object;

			if ( null != reader.getPixels() ) {
				try {
					int idx = 0;
					reader.getPixels()[ idx ] = reader.getPixels()[ idx ];
					idx = reader.getWidth() * reader.getHeight() - 1;
					reader.getPixels()[ idx ] = reader.getPixels()[ idx ];
					support = true;
				} catch ( Exception e ) {
				}
			}

		} catch ( Exception e ) {
		}
		return support;
	}

	//////////////////////////////////////////////////////////

	private PixelBufferReadInterface pixelBuffer;	
	private int w, h, max;
	private int pixels[];
	private int clearColor;
		
};

/**
 *
 * $Log: BufferFx.java,v $
 * Revision 1.4  2005/03/19 17:50:02  brian
 * repackaging
 *
 * Revision 1.3  2004/09/02 23:32:26  brian
 * dont make the RendererInterface extend PixelBufferInterface
 *
 * Revision 1.2  2004/09/02 15:43:46  brian
 * redefine PixelBufferReadInterface so BufferFx can use it as an interface rather than the RenderInterface
 *
 * Revision 1.1  2004/09/02 13:17:16  brian
 * the big reorg
 *
 * Revision 1.8  2004/09/01 01:10:42  brian
 * fix class level javadoc placement
 *
 * Revision 1.7  2004/09/01 00:11:06  brian
 * author, log and header stuff
 *
 *
 */
