package com.insanityengine.ghia.pixels;

/**
 *
 * <P>
 * This interface is meant to represent an MemoryImageSource kinda 
 * guy with some pretty accessors. The reads live over in the
 * PixelBufferReadInterface....
 * </P>
 *
 * <P>
 * It is good fun and you should see PixelImage for an example.
 * </P>
 *
 * @author BrianHammond
 *
 * $Header: /usr/local/cvsroot/ghia/src/java/com/insanityengine/ghia/pixels/PixelBufferInterface.java,v 1.3 2005/03/19 17:50:02 brian Exp $
 *
 */

public interface PixelBufferInterface extends PixelBufferReadInterface {

	/**
	 *
	 * Set the pixel at idx to color
	 *
	 * @param idx   pixel to set
	 * @param color use
	 *
	 */
	public void setColor( int idx, int color );
	
	/**
	 *
	 * Set the pixel at (x,y) to color
	 *
	 * @param x coordinate
	 * @param y coordinate
	 * @param color use
	 *
	 */
	public void setColor( int x, int y, int color );
	
	/**
	 *
	 * Set the pixel at (x,y) to color
	 *
	 * @param x coordinate
	 * @param y coordinate
	 * @param alpha component of the color
	 * @param red component of the color
	 * @param green component of the color
	 * @param blue component of the color
	 *
	 */
	public void setColor( int x, int y, int alpha, int red, int green, int blue );

	/**
	 *
	 * Set the clear color
	 *
	 * @return color to clear with
	 *
	 */
	public void setClearColor( int color );
	
	/**
	 *
	 * Clear the buffer
	 *
	 */
	public void clear();

	/**
	 *
	 * Copy the contents of the pixel to the graphic g
	 *
	 * @param graphic to copy to
	 *
	 */
	public void copyTo( java.awt.Graphics graphic );

	/**
	 *
	 * Set the image observer
	 *
	 * @param observer to use
	 *
	 */
	public void setObserver( java.awt.image.ImageObserver observer );
	
};

/**
 *
 * $Log: PixelBufferInterface.java,v $
 * Revision 1.3  2005/03/19 17:50:02  brian
 * repackaging
 *
 * Revision 1.2  2004/09/02 15:43:46  brian
 * redefine PixelBufferReadInterface so BufferFx can use it as an interface rather than the RenderInterface
 *
 * Revision 1.1  2004/09/02 13:17:16  brian
 * the big reorg
 *
 * Revision 1.5  2004/09/01 02:18:56  brian
 * javadoc
 *
 * Revision 1.4  2004/09/01 01:10:42  brian
 * fix class level javadoc placement
 *
 * Revision 1.3  2004/09/01 00:11:06  brian
 * author, log and header stuff
 *
 *
 */
