package com.insanityengine.ghia.pixels;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.applet.*;

/**
 *
 * <P>
 * Pretty much like MemoryImageSource, but without the lame constructor
 * signature... 
 * <img src="http://www.dlogic.org/~vic/images/smileys/yahoo_tongue.gif"/>
 * </P>
 *
 * @author BrianHammond
 *
 * $Header: /usr/local/cvsroot/ghia/src/java/com/insanityengine/ghia/pixels/PixelImage.java,v 1.4 2005/03/27 00:25:14 brian Exp $
 *
 */

public class PixelImage implements ImageProducer, PixelBufferInterface {

	/**	
	 *
	 * Constructor
	 *
	 */
	public PixelImage() {}

	/**	
	 *
	 * Constructor
	 *
	 * @param width of the image
	 * @param height of the image
	 *
	 */
	public PixelImage( int width, int height ) {
		init( width, height );
	}

	/**	
	 *
	 * Initialize this PixelImage
	 *
	 * @param width of the image
	 * @param height of the image
	 *
	 */
	public void init( int width, int height ) {
		this.w = width; 
		this.h = height;

		xm = w / 2;
		ym = h / 2;
		
		max = w * h;
		pixels = new int[ w * h ];
		memimg = Toolkit.getDefaultToolkit().createImage( this );
	}
	
	/**	
	 *
	 * Set the ImageObserver
	 *
	 * @param observer to use
	 *
	 */
	public void setObserver( ImageObserver observer ) {
		observ = observer;
	}
	
	/**
	 *
	 * Determines the width of the buffer. 
	 *
	 * @return the width of this buffer
	 *
	 */
	public int getWidth() {
		return w;
	}

	/**
	 *
	 * Determines the height of the buffer. 
	 *
	 * @return the height of this buffer
	 *
	 */
	public int getHeight() {
		return h;
	}

	/**
	 *
	 * Get the pixels
	 *
	 * @return the pixels
	 *
	 */
	public final int[] getPixels() {
		return pixels;
	}
	
	/**
	 *
	 * Set the pixel at idx to color
	 *
	 * @param idx   pixel to set
	 * @param color use
	 *
	 */
	public final void setColor( int idx, int color ) {
		pixels[ idx ] = color;
	}
	
	/**
	 *
	 * Set the pixel at (x,y) to color
	 *
	 * @param x coordinate
	 * @param y coordinate
	 * @param color use
	 *
	 */
	public final void setColor( int x, int y, int color ) {
		setColor( x + y * w, color );
	}
	
	/**
	 *
	 * Set the pixel at (x,y) to color
	 *
	 * @param x coordinate
	 * @param y coordinate
	 * @param alpha component of the color
	 * @param red component of the color
	 * @param green component of the color
	 * @param blue component of the color
	 *
	 */
	public final void setColor( int x, int y, int alpha, int red, int green, int blue ) {
		setColor( 
			x + y * w,
			alpha << 24 | red << 16 | green << 8 | blue
		);
	}
	
	/**
	 *
	 * Clear the buffer
	 *
	 */
	public void clear() {
		int cleared = 1;
		pixels[ 0 ] = clearColor;
		for ( cleared = 1 ; cleared < max / 2 ; cleared *= 2 ) {
			System.arraycopy( pixels, 0, pixels, cleared, cleared );
		}
		System.arraycopy( pixels, 0, pixels, cleared, max - cleared );
	}

	/**
	 *
	 * Copy the contents of the pixel to the graphic g
	 *
	 * @param graphic to copy to
	 *
	 */
	public void copyTo( Graphics graphic ) {
		update();
		graphic.drawImage( memimg, 0, 0, observ );
	}
	
	/**
	 *
	 * Set the clear color
	 *
	 * @return color to clear with
	 *
	 */
	public void setClearColor( int color ) { 
		clearColor = color; 
	}

	/**
	 *
	 * Get the clear color
	 *
	 * @return the clear color
	 *
	 */
	public int getClearColor() {
		return clearColor;
	}

	////////////////////////////////////////////////////////////////////
	// ImageProducer madness...
	////////////////////////////////////////////////////////////////////
	
	private ImageConsumer con = null;
	
	public synchronized void addConsumer( ImageConsumer c ) { con = c; }
	
	public final void startProduction( ImageConsumer c ) {
		if ( con != c ) {
			con = c;
			con.setDimensions( w ,h );
			con.setProperties( null );
			con.setColorModel( cm );
			con.setHints( ip_hints );
		}
		con.setPixels( 0, 0, w, h, cm, pixels, 0, w );
		con.imageComplete( ip_sfd );
	}
	
	public final void update() {
		if ( null != con ) startProduction( con );
	}
	
	public final boolean isConsumer( ImageConsumer c ) { return ( con == c ); }
	
	public final void requestTopDownLeftRightResend( ImageConsumer c ) {}
	
	public final void removeConsumer( ImageConsumer c ) {}	
	
	////////////////////////////////////////////////////////////////////
	// ImageProducer madness...
	////////////////////////////////////////////////////////////////////

	private static ColorModel cm = new DirectColorModel( 32, 0xFF0000, 0x00FF00, 0x0000FF );
	private static int ip_hints = (
		ImageConsumer.TOPDOWNLEFTRIGHT |
		ImageConsumer.COMPLETESCANLINES |
		ImageConsumer.SINGLEPASS |
		ImageConsumer.SINGLEFRAME
	);
	private static int ip_sfd = ImageConsumer.SINGLEFRAMEDONE;
	
	//////////////////

	protected int w = 0;
	protected int h = 0;
	protected int pixels[] = null;

	int xm = 0;
	int ym = 0;
	int max = 0;
	
	private ImageObserver observ = null;

	Image memimg = null;

	int clearColor = 0;

};

/**
 *
 * $Log: PixelImage.java,v $
 * Revision 1.4  2005/03/27 00:25:14  brian
 * www not elvis
 *
 * Revision 1.3  2005/03/27 00:14:11  brian
 * fix image
 *
 * Revision 1.2  2005/03/19 17:50:02  brian
 * repackaging
 *
 * Revision 1.1  2004/09/02 13:17:16  brian
 * the big reorg
 *
 * Revision 1.8  2004/09/01 14:53:53  brian
 * more javadoc
 *
 * Revision 1.7  2004/09/01 02:18:56  brian
 * javadoc
 *
 * Revision 1.6  2004/09/01 01:10:42  brian
 * fix class level javadoc placement
 *
 * Revision 1.5  2004/09/01 00:11:06  brian
 * author, log and header stuff
 *
 *
 */
