package com.insanityengine.ghia.renderer;

import com.insanityengine.ghia.m3.*;
import com.insanityengine.ghia.pixels.*;
import com.insanityengine.ghia.libograf.*;

/**
 *
 * <P>
 * The "default" Renderer and some factory style methods
 * </P>
 *
 * @author BrianHammond
 *
 * $Header: /usr/local/cvsroot/ghia/src/java/com/insanityengine/ghia/renderer/Renderer.java,v 1.4 2005/03/19 17:50:02 brian Exp $
 *
 */

public class Renderer extends Memagery {

	/**
	 * 
	 * Try to create a renderer given the name of the class 
	 * It will be initialized with the values passed into the init
	 * method of this LiboGraf instance
	 *
	 * @param nameOfRenderer to try to use
	 *
	 * @return instance of the renderer on success, null on fail
	 *
	 */
	public final static RendererInterface getRendererByName( String nameOfRenderer ) {

		String cause = "uh...";
		RendererInterface yourRenderer = null;

		Class theClass = com.insanityengine.ghia.util.DocReflect.forName( nameOfRenderer );
		if ( null == theClass ) {
			cause = "class for name failed for " + nameOfRenderer;
		} else {

			String interfaceName = "com.insanityengine.ghia.renderer.RendererInterface";
			Class[] interfaces = theClass.getInterfaces();
			cause = nameOfRenderer + " does not implement " + interfaceName;

			try {
				yourRenderer = ( RendererInterface ) theClass.newInstance();
			} catch ( Exception e ) {
				e.printStackTrace();
				cause = "didn't you see the stacktrace?";
			}
		}

		if ( null == yourRenderer ) System.err.println( "Renderer.getRendererByName failed: " + cause );
		else System.err.println( "Renderer.getRendererByName succeeded: " + nameOfRenderer );

		return yourRenderer;
	}

	/**
	 *
	 * Get an instance of the default RenderInterface implementation
	 *
	 * @return an instance of the default RenderInterface implementation
	 *
	 */
	public final static RendererInterface getDefaultRenderer() {
		return new Renderer();
	}
};

/**
 *
 * $Log: Renderer.java,v $
 * Revision 1.4  2005/03/19 17:50:02  brian
 * repackaging
 *
 * Revision 1.3  2004/09/11 16:22:26  brian
 * added getDefaultRenderer static method
 *
 * Revision 1.2  2004/09/11 14:57:29  brian
 * updated status message
 *
 * Revision 1.1  2004/09/11 14:52:23  brian
 * 'default renderer' mechanism and class for name static method
 *
 *
 */
