package com.insanityengine.ghia.renderer;

import java.util.Stack;

import java.awt.*;

import java.applet.Applet;
import java.awt.Component;
import java.awt.image.ImageObserver;

import com.insanityengine.ghia.m3.*;
import com.insanityengine.ghia.libograf.*;
import com.insanityengine.ghia.pixels.*;
import com.insanityengine.ghia.renderer.*;

/**
 *
 * <P>
 * Implements the RendererConsumerInterface.
 * </P>
 *
 * @author BrianHammond
 *
 * $Header: /usr/local/cvsroot/ghia/src/java/com/insanityengine/ghia/renderer/RendererConsumer.java,v 1.6 2006/11/06 19:47:42 brian Exp $
 *
 */

public class RendererConsumer implements RendererConsumerInterface {

	////////////////////////////////////////////////////////

	/**
	 *
	 * Constructor
	 *
	 */
	public RendererConsumer() {
		init( 0, 0, null );
	}

	/**
	 *
	 * Constructor
	 *
	 * @param width of the display
	 * @param height of the display
	 * @param component to use for graphics (yuck!)
	 *
	 */
	public RendererConsumer( int width, int height, java.awt.Component component ) {
		init( width, height, component );
	}

	/**
	 *
	 * Constructor, if m3_renderName parameter is defined for the 
	 * Applet, it will attempt to use it with the createRenderer
	 * method
	 *
	 * @param applet which provides context
	 *
	 */
	public RendererConsumer( Applet applet ) {
		init( applet );
	}

	/**
	 *
	 * Initialize this RendererConsumer
	 *
	 * @param applet which provides context
	 *
	 */
	public void init( Applet applet ) {
		init( 
			applet.getSize().width,
			applet.getSize().height,
			applet
		);
		renderer.setDepthTesting( true );
		renderer.setShadingMethod( M3_Constants.NONE_SHADE );
		renderer.setShadingMethod( M3_Constants.PHONG_SHADE );
		renderer.setShadingMethod( M3_Constants.NORM_SHADE );
	}
	
	/**
	 *
	 * Initialize this RendererConsumer
	 *
	 * @param width of the display
	 * @param height of the display
	 * @param component to use
	 *
	 */
	public void init( int width, int height, java.awt.Component component ) {

		w = width;
		h = height;
		this.component = component;

		if ( 0 != width ) createRenderer();
		if ( null == renderer ) {
			this.setRenderer( new com.insanityengine.ghia.renderer.Abrashed() );
		}
		if ( null != renderer ) renderer.setObserver( component );
	}

	/**
	 *
	 * Create a renderer with a given size
	 *
	 * @param width of the renderer to create
	 * @param height of the renderer to create
	 *
	 * @return true on success, false on fail
	 *
	 */
	public final boolean createRenderer( int width, int height ) {
		w = width;
		h = height;
		return createRenderer();
	}

	/**
	 * 
	 * Try to create a renderer given the name of the class 
	 * It will be initialized with the values passed into the init
	 * method of this LiboGraf instance
	 *
	 * @param nameOfRenderer to try to use
	 *
	 * @return true if succeds, else false
	 *
	 */
	public final boolean createRenderer( String nameOfRenderer ) {
		
		String cause = "unknown";
		boolean success = false;

		RendererInterface renderer = Renderer.getRendererByName( nameOfRenderer );
		if ( null == renderer ) {
			cause = "no rendererer named " + nameOfRenderer;
		} else {
			renderer.init( w, h, component );
			this.renderer = renderer;
			success = true;
		}

		if ( !success ) System.err.println( "RendererConsumer.createRenderer failed: " + cause );
		else System.err.println( "RendererConsumer.createRenderer succeeded: " + nameOfRenderer );
		
		return success;
	}

	/**
	 *
	 * Create a renderer of the default type using the values from 
	 * sent to init
	 *
	 * @return true if succeds, else false
	 *
	 */
	public final boolean createRenderer() {
		String rname = null;
		String pname = "m3_renderName";
		
		if ( component instanceof Applet ) {
			try {
				rname = ( ( Applet ) component ).getParameter( pname );
			} catch ( Exception e ) {
			}
		}
	
		try {	
			if ( null == rname ) rname = System.getProperty( pname );
		} catch ( Exception e ) {
		}
		
		if ( null == rname ) rname = "com.insanityengine.ghia.renderer.Memagery";
		return createRenderer( rname );
	}
	
	/**
	 *
	 * Get the renderer
	 *
	 * @return the renderer
	 *
	 */
	public final RendererInterface getRenderer() { 
		return renderer;
	}

	/**
	 *
	 * Set the renderer
	 *
	 * @param renderer to use
	 *
	 */
	public final void setRenderer( RendererInterface renderer ) {
		this.renderer = renderer;
		w = renderer.getWidth();
		h = renderer.getHeight();
	}

	/**
	 *
	 * Render the scene
	 *
	 */
	public void render( LiboGraf gl ) {
		renderer.render( gl );
	}

	/**
	 *
	 * Set the graphics to use
	 * 
	 * @param graphics to use
	 *
	 */
	public void setGraphics( Graphics graphics ) {
		this.graphics = graphics;
	}
	
	/**
	 *
	 * Get the graphics
	 *
	 */
	public Graphics getGraphics() {
		return graphics;
	}
	
	////////////////////////////////////////////////////////
	
	protected int w = 0, h = 0;
	private RendererInterface renderer = null;
	private Component component = null;
	private Graphics graphics = null;
	
};

/**
 *
 * $Log: RendererConsumer.java,v $
 * Revision 1.6  2006/11/06 19:47:42  brian
 * set default renderer
 *
 * Revision 1.5  2005/03/19 17:50:02  brian
 * repackaging
 *
 * Revision 1.4  2004/09/11 14:56:06  brian
 * use the Renderer.getRendererByName static method
 *
 * Revision 1.3  2004/09/11 14:44:59  brian
 * each class should have an interface and vice versa...
 *
 * Revision 1.2  2004/09/11 14:37:21  brian
 * added RendererConsumer and stuff
 *
 */
