package com.insanityengine.ghia.renderer;

import com.insanityengine.ghia.m3.*;
import com.insanityengine.ghia.pixels.*;
import com.insanityengine.ghia.libograf.*;

/**
 *
 * <P>
 * Attempts to define an interface for rendering polygons
 * </P>
 *
 * @author BrianHammond
 *
 * $Header: /usr/local/cvsroot/ghia/src/java/com/insanityengine/ghia/renderer/RendererInterface.java,v 1.5 2005/03/19 17:50:02 brian Exp $
 *
 */

public interface RendererInterface extends DrawerRegistryInterface { 

	/**
	 *
	 * Invoke any drawing routines etc
	 *
	 * @param context to use to render
	 *
	 */
	public void render( LiboGraf context );

	/**
	 *
	 * Initialize the RendererInterface implementation. These 
	 * arguments should be sufficient for all implementations (!!!), 
	 * but may need to be made more extensible (RendererContextInterface?)
	 *
	 * @param width of the display
	 * @param height of the display
	 * @param component to use to create offscreen buffer
	 *
	 */
	public void init( int width, int height, java.awt.Component component );

	/**
	 *
	 * Draw a polygon
	 *
	 * @param count of points
	 * @param ptz the array of Pt3 to draw
	 *
	 */
	public void drawPolygon( int count, Pt3[] ptz );
	
	/**
	 *
	 * Get the clear color
	 *
	 * @return the clear color
	 *
	 */
	public int getClearColor();

	/**
	 *
	 * Set the clear color
	 *
	 * @return color to clear with
	 *
	 */
	public void setClearColor( int color );

	/**
	 *
	 * Set depth testing policy
	 *
	 * @param testDepth false disables test
	 *
	 */
	public void setDepthTesting( boolean testDepth );

	/**
	 *
	 * Clear the depth buffer
	 *
	 */
	public void clearDepthBuffer();

	/**
	 *
	 * Clear the pixel/color buffer
	 *
	 */
	public void clearColorBuffer();

	/**
	 *
	 * Set the shading method
	 *
	 * @param method may be listed in M3_Constants
	 *
	 */
	public void setShadingMethod( int method );

	/**
	 *
	 * Set the texture function
	 *
	 * @param function may be listed in M3_Constants
	 *
	 */
	public void setTextureFunction( int function );

	/**
	 *
	 * Set the current normal 
	 *
	 * @param point to use as the current normal
	 *
	 */
	public void setNormal( Pt3 point );
	
	/**
	 *
	 * Set the current ImageSkin
	 *
	 * @param imageskin to use
	 *
	 */
	public void setSkin( ImageSkin skin );

	/********************************************************************/

	/**
	 *
	 * Set the image observer
	 *
	 * @param observer to use
	 *
	 */
	public void setObserver( java.awt.image.ImageObserver observer );

	/**
	 *
	 * Determines the width of the buffer. 
	 *
	 * @return the width of this buffer
	 *
	 */
	public int getWidth();

	/**
	 *
	 * Determines the height of the buffer. 
	 *
	 * @return the height of this buffer
	 *
	 */
	public int getHeight();

	/**
	 *
	 * Clear the buffer
	 *
	 */
	public void clear();

};

/**
 *
 * $Log: RendererInterface.java,v $
 * Revision 1.5  2005/03/19 17:50:02  brian
 * repackaging
 *
 * Revision 1.4  2004/09/11 14:37:21  brian
 * added RendererConsumer and stuff
 *
 * Revision 1.3  2004/09/05 03:41:31  brian
 * refactored to have the render method of the renderers invoke the drawingIfz
 *
 * Revision 1.2  2004/09/02 23:32:26  brian
 * dont make the RendererInterface extend PixelBufferInterface
 *
 * Revision 1.1  2004/09/02 13:17:16  brian
 * the big reorg
 *
 * Revision 1.4  2004/09/01 01:10:42  brian
 * fix class level javadoc placement
 *
 * Revision 1.3  2004/09/01 00:11:06  brian
 * author, log and header stuff
 *
 *
 */
