package com.insanityengine.ghia.renderer;

import java.awt.*;
import java.awt.image.*;

import com.insanityengine.ghia.m3.*;
import com.insanityengine.ghia.pixels.*;
import com.insanityengine.ghia.libograf.*;

/**
 *
 * <P>
 * Super cheap wireframe implementation of RendererInterface.
 * </P>
 *
 * <table border="1">
 *  <topic>TestResults</topic>
 *  <TH>TestName</TH>
 *  <TH>Results</TH>
 *  <TH>SnapShot</TH>
 *  <TR valign="top">
 *   <TD>Light</TD>
 *   <TD>1593 fps</TD>
 *   <TD><img src="http://ghia.insanityengine.com/images/old_snaps/WireFuRender_Light.gif"/></TD>
 *  </TR>
 *  <TR valign="top">
 *   <TD>GeomLoader</TD>
 *   <TD>2370/60.817=38.969368 * 4725 = 184130 tps = 184 kts</TD>
 *   <TD><img src="http://ghia.insanityengine.com/images/old_snaps/WireFuRender_Geom.gif"/></TD>
 *  </TR>
 * </TABLE>
 *
 * @author BrianHammond
 *
 * $Header: /usr/local/cvsroot/ghia/src/java/com/insanityengine/ghia/renderer/WireFuRenderer.java,v 1.8 2005/03/26 23:42:41 brian Exp $
 *
 */

public class WireFuRenderer implements RendererInterface, PixelBufferReadInterface {

	/** 
	 * 
	 * Constructor
	 * 
	 */
	public WireFuRenderer() {
	}

	/** 
	 * 
	 * Constructor
	 *
	 * @param width of the display
	 * @param height of the display
	 * @param component to use to create offscreen buffer
	 *
	 */
	public WireFuRenderer( int width, int height, Component component ) {
		init( width, height, component );
	}

	/**
	 *
	 * Initialize this WireFuRenderer, call this *after* the component
	 * has been made visible or you will suffer!
	 *
	 * @param width of the display
	 * @param height of the display
	 * @param component to use to create offscreen buffer
	 *
	 */
	public void init( int width, int height, Component component ) {
		setObserver( component );
		
		w = width;
		h = height;
		frustrum.setSize( w, h );

		image = null;
		this.component = component;
	}
	
	/**
	 *
	 * Render the next scene
	 * 
	 * @param gl to use to render
	 *
	 */
	public void render( LiboGraf gl ) {
		if ( graphicsReady() ) {
			invokeDrawers( gl );
			gl.getGraphics().drawImage( image, 0, 0, observer );
		}
	}
	
	/**
	 *
	 * Get the clear color
	 *
	 * @return the clear color
	 *
	 */
	public int getClearColor() {
		return 0;
	}

	/**
	 *
	 * Set the clear color
	 *
	 * @return color to clear with
	 *
	 */
	public void setClearColor( int color ) {
	}

	/**
	 *
	 * Set depth testing policy
	 *
	 * @param testDepth false disables test
	 *
	 */
	public void setDepthTesting( boolean testDepth ) { }

	/**
	 *
	 * Clear the depth buffer
	 *
	 */
	public void clearDepthBuffer() { 
	}

	/**
	 *
	 * Clear the pixel/color buffer
	 *
	 */
	public void clearColorBuffer() {
		clear();
	}

	/**
	 *
	 * Set the shading method
	 *
	 * @param method may be listed in M3_Constants
	 *
	 */
	public void setShadingMethod( int method ) {
	}

	/**
	 *
	 * Set the texture function
	 *
	 * @param function may be listed in M3_Constants
	 *
	 */
	public void setTextureFunction( int function ) {
	}

	/**
	 *
	 * Draw a polygon
	 *
	 * @param count of points
	 * @param ptz the array of Pt3 to draw
	 *
	 */
	public void drawPolygon( int count, Pt3[] ptz ) {
		if ( graphicsReady() ) {
		
			frustrum.toScreen( count, ptz );
			
			Polygon poly = new Polygon();
			graphics.setColor( Color.green );
			for ( int i = 0 ; i < count ; ++i ) {
				poly.addPoint( ( int ) ptz[ i ].x , ( int ) ptz[ i ].y );
			}
			graphics.drawPolygon( poly );
		}
	}

	/**
	 *
	 * Set the current normal 
	 *
	 * @param point to use as the current normal
	 *
	 */
	public void setNormal( Pt3 point ) {
	}
	
	/**
	 *
	 * Set the current ImageSkin
	 *
	 * @param imageskin to use
	 *
	 */
	public void setSkin( ImageSkin skin ) {
	}

	///////////////////////////////////////////////////////////////////////
	// PixelBufferInterface
	
	/**
	 *
	 * Clear the buffer
	 *
	 */
	public void clear() {
		if ( graphicsReady() ) {
			graphics.setColor( Color.black );
			graphics.fillRect( 0, 0, w, h );
		}
	}

	/**
	 *
	 * Set the image observer
	 *
	 * @param observer to use
	 *
	 */
	public void setObserver( java.awt.image.ImageObserver observer ) {
		this.observer = observer;
	}

	/**
	 *
	 * Determines the width of the buffer. 
	 *
	 * @return the width of this buffer
	 *
	 */
	public int getWidth() {
		return w;
	}

	/**
	 *
	 * Determines the height of the buffer. 
	 *
	 * @return the height of this buffer
	 *
	 */
	public int getHeight() {
		return h;
	}

	/**
	 *
	 * Because this uses a PixelGrabber it may be slow-ish...
	 *
	 * @return array of pixel data
	 *
	 */
	public int[] getPixels() {
		ImageSkin skin = new ImageSkin( image, observer );
		return skin.getPixels();
	}
	
	//////////////////////////////////////////////////
	// DrawerRegistryInterface
	
	/**
	 *
	 * Some renderers are big babys about when it is ok to mess with their goodies.
	 * Hence this cheesy add/remove Drawer mess
	 *
	 * @param drawer to call per frame
	 *
	 */
	public void addDrawer( DrawingInterface drawer ) {
		registry.addDrawer( drawer );
	}

	/**
	 *
	 * Some renderers are big babys about when it is ok to mess with their goodies.
	 * Hence this cheesy add/remove Drawer mess
	 *
	 * @param drawer to call per frame
	 *
	 */
	public void removeDrawer( DrawingInterface drawer ) {
		registry.removeDrawer( drawer );
	}

	/**
	 *
	 * Invoke any registered DrawingInterfaces
	 *
	 */
	public void invokeDrawers( LiboGraf gl ) {
		registry.invokeDrawers( gl );
	}

	///////////////////////////////////////////////////

	private int w, h;
	private Image image = null; 
	private Graphics graphics = null;
	private ImageObserver observer = null;
	private Component component = null;
	private DrawerRegistry registry = new DrawerRegistry();
	private Frustrum frustrum = new Frustrum();
	
	///////////////////////////////////////////////////
	
	private final boolean graphicsReady() {
		boolean ok = false;
		if ( null == graphics ) {
			image = component.createImage( w, h );
			if ( null == image ) {
			} else {
				ok = true;
				graphics = image.getGraphics();
			}
			System.out.println( "" + " image = " + image  + " graphics = " + graphics );
		} else {
			ok = true;
		}
		return ok;
	}

};

/**
 *
 * $Log: WireFuRenderer.java,v $
 * Revision 1.8  2005/03/26 23:42:41  brian
 * fix links
 *
 * Revision 1.7  2005/03/19 17:50:02  brian
 * repackaging
 *
 * Revision 1.6  2004/09/16 11:00:20  brian
 * added expensive getPixels impl... but how to mark it as such programmaticly?
 *
 * Revision 1.5  2004/09/13 03:06:41  brian
 * fix up Frustrum.toScreen, still working on JoglRenderer's texture support... sigh...
 *
 * Revision 1.4  2004/09/05 03:41:31  brian
 * refactored to have the render method of the renderers invoke the drawingIfz
 *
 * Revision 1.3  2004/09/03 03:38:13  brian
 * fixed typo
 *
 * Revision 1.2  2004/09/03 02:42:12  brian
 * remove cruft
 *
 * Revision 1.1  2004/09/02 13:17:16  brian
 * the big reorg
 *
 * Revision 1.6  2004/09/01 02:18:57  brian
 * javadoc
 *
 * Revision 1.5  2004/09/01 01:52:47  brian
 * added test results (ok, and images) to javadoc
 *
 * Revision 1.4  2004/09/01 01:10:42  brian
 * fix class level javadoc placement
 *
 * Revision 1.3  2004/09/01 00:11:06  brian
 * author, log and header stuff
 *
 *
 */
