package com.insanityengine.ghia.renderer;

import java.awt.*;
import java.awt.image.*;

import java.util.*;

import com.insanityengine.ghia.m3.*;
import com.insanityengine.ghia.libograf.*;
import com.insanityengine.ghia.pixels.*;

import com.insanityengine.ghia.util.*;

/**
 *
 * <P>
 * This implementation of RendererInterface uses a zsorted list of polygons to make your
 * fall in love with it! And you will!  Everyone will mock you because this peice of code
 * holds your fancy! Bwhaahahahahhahaha!
 * </P>
 *
 * <table border="1">
 *  <topic>TestResults</topic>
 *  <TH>TestName</TH>
 *  <TH>Results</TH>
 *  <TH>SnapShot</TH>
 *  <TR valign="top">
 *   <TD>Light</TD>
 *   <TD>1646 fps</TD>
 *   <TD><img src="http://ghia.insanityengine.com/images/old_snaps/ZortPolyRenderer_Light.gif"/></TD>
 *  </TR>
 *  <TR valign="top">
 *   <TD>GeomLoader</TD>
 *   <TD>97510/60.306=1616.9204 * 4725 = 7639948 tps = 76 kts</TD>
 *   <TD><img src="http://ghia.insanityengine.com/images/old_snaps/ZortPolyRenderer_Geom.gif"/></TD>
 *  </TR>
 * </TABLE>
 *
 * @author BrianHammond
 *
 * $Header: /usr/local/cvsroot/ghia/src/java/com/insanityengine/ghia/renderer/ZortPolyRenderer.java,v 1.8 2005/03/26 23:42:41 brian Exp $
 *
 */

public class ZortPolyRenderer implements RendererInterface, PixelBufferReadInterface {

	/** 
	 * 
	 * Constructor
	 * 
	 */
	public ZortPolyRenderer() {
		initColors();
	}

	/** 
	 * 
	 * Constructor
	 *
	 * @param width of the display
	 * @param height of the display
	 * @param component to use to create offscreen buffer
	 *
	 */
	public ZortPolyRenderer( int width, int height, Component component ) {
		initColors();
		init( width, height, component );
	}

	/**
	 *
	 * Initialize this ZortPolyRenderer, call this *after* the component
	 * has been made visible or you will suffer!
	 *
	 * @param width of the display
	 * @param height of the display
	 * @param component to use to create offscreen buffer
	 *
	 */
	public void init( int width, int height, Component component ) {
		setObserver( component );
		
		w = width;
		h = height;

		frustrum.setSize( width, height );

		image = null;
		this.component = component;
	}
	
	/**
	 *
	 * Render the next scene
	 * 
	 * @param gl to use to render
	 *
	 */
	public void render( LiboGraf gl ) {
		if ( graphicsReady() ) {
			invokeDrawers( gl );
			happyPants( graphics );
			gl.getGraphics().drawImage( image, 0, 0, observer );
		}
	}
	
	/**
	 *
	 * Get the clear color
	 *
	 * @return the clear color
	 *
	 */
	public int getClearColor() {
		return 0;
	}

	/**
	 *
	 * Set the clear color
	 *
	 * @return color to clear with
	 *
	 */
	public void setClearColor( int color ) {
	}

	/**
	 *
	 * Set depth testing policy
	 *
	 * @param testDepth false disables test
	 *
	 */
	public void setDepthTesting( boolean testDepth ) { }

	/**
	 *
	 * Clear the depth buffer
	 *
	 */
	public void clearDepthBuffer() { 
	}

	/**
	 *
	 * Clear the pixel/color buffer
	 *
	 */
	public void clearColorBuffer() {
		clear();
	}

	/**
	 *
	 * Set the shading method
	 *
	 * @param method may be listed in M3_Constants
	 *
	 */
	public void setShadingMethod( int method ) {
	}

	/**
	 *
	 * Set the texture function
	 *
	 * @param function may be listed in M3_Constants
	 *
	 */
	public void setTextureFunction( int function ) {
	}

	/**
	 *
	 * Draw a polygon
	 *
	 * @param count of points
	 * @param ptz the array of Pt3 to draw
	 *
	 */
	public void drawPolygon( int count, Pt3[] ptz ) {

		if ( graphicsReady() ) {
			frustrum.toScreen( count, ptz );

			if ( zcurr > 0 && !frustrum.offScreen( count, ptz ) ) {
				if ( true ) {
				
					polyz.add( new PrettyPoly( count, ptz, zcurr ) );
					
				} else {
					Polygon poly = mkPolygon( count, ptz );

					int c = zToC( zcurr );
					c = 128 + c / 2;
					graphics.setColor( new Color( c, c, c ) );
					graphics.fillPolygon( poly );
				}
			}

		}
	}

	/**
	 *
	 * Set the current normal 
	 *
	 * @param point to use as the current normal
	 *
	 */
	public void setNormal( Pt3 point ) {
		zcurr = point.z;
		normal.set( point );
	}
	
	/**
	 *
	 * Set the current ImageSkin
	 *
	 * @param imageskin to use
	 *
	 */
	public void setSkin( ImageSkin skin ) {
	}

	///////////////////////////////////////////////////////////////////////
	// PixelBufferInterface
	
	/**
	 *
	 * Clear the buffer
	 *
	 */
	public void clear() {
		if ( graphicsReady() ) {
			graphics.setColor( Color.black );
			graphics.fillRect( 0, 0, w, h );
		}
	}

	/**
	 *
	 * Set the image observer
	 *
	 * @param observer to use
	 *
	 */
	public void setObserver( java.awt.image.ImageObserver observer ) {
		this.observer = observer;
	}

	/**
	 *
	 * Determines the width of the buffer. 
	 *
	 * @return the width of this buffer
	 *
	 */
	public int getWidth() {
		return w;
	}

	/**
	 *
	 * Determines the height of the buffer. 
	 *
	 * @return the height of this buffer
	 *
	 */
	public int getHeight() {
		return h;
	}
	
	/**
	 *
	 * Because this uses a PixelGrabber it may be slow-ish...
	 *
	 * @return array of pixel data
	 *
	 */
	public int[] getPixels() {
		ImageSkin skin = new ImageSkin( image, observer );
		return skin.getPixels();
	}
	
	//////////////////////////////////////////////////
	// DrawerRegistryInterface
	
	/**
	 *
	 * Some renderers are big babys about when it is ok to mess with their goodies.
	 * Hence this cheesy add/remove Drawer mess
	 *
	 * @param drawer to call per frame
	 *
	 */
	public void addDrawer( DrawingInterface drawer ) {
		registry.addDrawer( drawer );
	}

	/**
	 *
	 * Some renderers are big babys about when it is ok to mess with their goodies.
	 * Hence this cheesy add/remove Drawer mess
	 *
	 * @param drawer to call per frame
	 *
	 */
	public void removeDrawer( DrawingInterface drawer ) {
		registry.removeDrawer( drawer );
	}

	/**
	 *
	 * Invoke any registered DrawingInterfaces
	 *
	 */
	public void invokeDrawers( LiboGraf gl ) {
		registry.invokeDrawers( gl );
	}

	///////////////////////////////////////////////////

	private int w, h;
	private Image image = null; 
	private Graphics graphics = null;
	private ImageObserver observer = null;
	private Component component = null;
	
	private DrawerRegistry registry = new DrawerRegistry();

	private Frustrum frustrum = new Frustrum();
	
	private ArrayList polyz = new ArrayList();
	private float zcurr = 1;
	private static Color [] colorz = null;

	private Pt3 normal = new Pt3();
	
	///////////////////////////////////////////////////
	
	private final boolean graphicsReady() {
		boolean ok = false;
		if ( null == graphics ) {
			image = component.createImage( w, h );
			if ( null == image ) {
			} else {
				ok = true;
				graphics = image.getGraphics();
			}
			System.out.println( "" + " image = " + image  + " graphics = " + graphics );
		} else {
			ok = true;
		}
		return ok;
	}

	private final void happyPants( LiboGraf gl ) {
		happyPants( gl.getGraphics() );
	}

	private final void happyPants( Graphics graphics ) {
		Collections.sort( polyz );
		int sz = polyz.size();
		for ( int i = 0 ; i < sz ; ++i ) {
			( ( PrettyPoly ) polyz.get( i ) ).draw( graphics );
		}
		polyz.clear();
	}

	private final static int zToC( float z ) {
		return 0 + ( int ) (
			255 * (
				( ( z + 0.4 ) * ( z + 0.4 ) / 1.96 )
			)
		);
	}

	private final static Color zToColor( int n ) {
		return colorz[ n ];
	}
	
	private final static Color zToColor( float z ) {
		return zToColor( zToC( z ) );
	}
	
	synchronized private final static void initColors() {
		if ( null == colorz ) {
			colorz = new Color[ 255 ];
			for ( int i = 0 ; i < 255 ; ++i ) {
				colorz[ i ] = new Color( i, i / 2, 255 - i );
				/*
				colorz[ i ] = new Color(
					  i * i / 255
					, i * i / 255
					, i * i / 255
				);
				*/
			}
		}
	}

	private static final Polygon mkPolygon( int count, Pt3[] ptz ) {
		Polygon poly = new Polygon();
		addToPolygon( poly, count, ptz );
		return poly;
	}
	
	private static final float addToPolygon( Polygon poly, int count, Pt3[] ptz ) {
		float zavg = 0;
		for ( int i = 0 ; i < count ; ++i ) {
			poly.addPoint( ( int ) ptz[ i ].x, ( int ) ptz[ i ].y );
			zavg += ptz[ i ].z;
		}
		return zavg / count;
	}

	private class PrettyPoly implements Comparable {
		
		public PrettyPoly() {
		}
		
		public PrettyPoly( int count, Pt3[] ptz ) {
			init( count, ptz, 1 );
		}
		
		public PrettyPoly( int count, Pt3[] ptz, float z ) {
			init( count, ptz, z );
		}

		public void init( int count, Pt3[] ptz, float z ) {
			this.poly = new Polygon();
			this.zpos = ZortPolyRenderer.addToPolygon( poly, count, ptz );
			this.z = z;

			x = y = 0;
			for ( int i = 0 ; i < count ; ++i ) {
				x += ( int ) ptz[ i ].x;
				y += ( int ) ptz[ i ].y;
			}
			x /= count;
			y /= count;
		}

		public void draw( Graphics graphics ) {
			graphics.setColor( ZortPolyRenderer.zToColor( z ) );
			graphics.fillPolygon( poly );
		}

		public int compareTo( Object o ) {
			return ( int ) ( 100 * zpos - ( ( PrettyPoly ) o ).zpos );
		}

		////

		private Polygon poly;
		private float z = 0; // normal.z
		private float zpos;

		int x, y;
	}
};

/**
 *
 * $Log: ZortPolyRenderer.java,v $
 * Revision 1.8  2005/03/26 23:42:41  brian
 * fix links
 *
 * Revision 1.7  2005/03/19 17:50:02  brian
 * repackaging
 *
 * Revision 1.6  2005/03/12 04:58:47  brian
 * call the methods set instead of copy
 *
 * Revision 1.5  2004/09/16 11:00:20  brian
 * added expensive getPixels impl... but how to mark it as such programmaticly?
 *
 * Revision 1.4  2004/09/11 14:37:21  brian
 * added RendererConsumer and stuff
 *
 * Revision 1.3  2004/09/09 03:07:51  brian
 * remove cool visual debugging
 *
 * Revision 1.2  2004/09/09 03:05:09  brian
 * better with tuns of wacky debug visuals you'll love
 *
 * Revision 1.1  2004/09/09 00:14:56  brian
 * barely works...
 *
 *
 */
