package com.insanityengine.ghia.util;

import java.io.File;
import java.util.Vector;
import java.util.Stack;

/**
 *
 * <P>
 * Towards a java replacement for the uberHandy UNIX find command
 * </P>
 *
 * <P>
 * % javadoc -d javadoc/ $( java -cp jfreak.jar com.insanityengine.ghia.util.Find -name "*.java" )
 * </P>
 *
 * @author BrianHammond
 *
 * $Header: /usr/local/cvsroot/ghia/src/java/com/insanityengine/ghia/util/Find.java,v 1.5 2005/03/19 17:50:02 brian Exp $
 *
 */

public class Find {
	
	/**
	 *
	 */
	public static void usage() {
		System.err.println( "boohoo" );
	}

	/**
	 *
	 */
	public final static void main( String argv[] ) {
		
		Vector directories = new Vector();
		Vector filters = new Vector();

		boolean dirs = true;
		for ( int i = 0 ; i < argv.length ; ++i ) {
			String arg = argv[ i ];

			if ( dirs ) {
				if ( '-' != arg.charAt( 0 ) ) {
					directories.addElement( arg );
					continue;
				}
			}

			if ( i == argv.length - 1 ) usage();
			
			String val = argv[ ++i ];

			FileFilter filter = getFilterByName( arg );
			if ( null == filter ) {
				System.err.println( arg + " not supported" );
				System.exit( 1 );
			} else {
				filter.init( val );
				filters.addElement( filter );
			}
		}

		find( directories, filters );
	}

	/**
	 *
	 */
	public final static Class forName( String name ) {
		Class lass = null;
		try {
			lass = Class.forName( name );
		} catch ( Exception e ) {
		}
		return lass;
	}

	

	/**
	 * like -name --> NameFileFilter --> com.insanityengine.ghia.util.NameFileFilter
	 */
	public final static FileFilter getFilterByName( String name ) {
		FileFilter filter = null;

		String interfaceName = "FileFilter";
		String packageName   = "com.insanityengine.ghia.util.";
		String fullName      = packageName + interfaceName;
		
		name = name.toUpperCase().charAt( 1 ) + name.substring( 2 ) + interfaceName;
		
		Class candidate = forName( name );
		if ( null == candidate ) {
			name = "com.insanityengine.ghia.util." + name;
			candidate  = forName( name );
		}

		if ( null != candidate ) {
			Class[] interfaces = candidate.getInterfaces();
			for ( int i = 0 ; i < interfaces.length ; ++i ) {
				if ( fullName.equals( interfaces[ i ].getName() ) ) {
					try {
						filter = ( FileFilter ) candidate.newInstance();
					} catch ( Exception e ) {
						e.printStackTrace();
					}
					break;
				}
			}

		}
	
		return filter;
	}
	
	/**
	 *
	 * Invoke a number of filters on a file until a filter fails
	 *
	 * @param file			to test
	 * @param filters		to invoke
	 *
	 * @return true if all filters pass, false otherwise
	 *
	 */
	public final static boolean passesFilters( File f, Vector filters ) {
		return invokeFilters( f, filters, true );
	}
	
	/**
	 *
	 * Invoke a number of filters on a file
	 *
	 * @param file			to test
	 * @param filters		to invoke
	 *
	 * @return true if all filters pass, false otherwise
	 *
	 */
	public final static boolean invokeFilters( File f, Vector filters ) {
		return invokeFilters( f, filters, false );
	}

	/**
	 *
	 * Invoke a number of filters on a file
	 *
	 * @param file			to test
	 * @param filters		to invoke
	 * @param bailOnFirst	if true, break out after first fail
	 *
	 * @return true if all filters pass, false otherwise
	 *
	 */
	public final static boolean invokeFilters( File file, Vector filters, boolean bailOnFirst ) {
		int fcount = filters.size();
		boolean rvalue = true;
		
		for ( int i = 0 ; i < fcount ; ++i ) {
			FileFilter filter = ( FileFilter ) filters.elementAt( i );
			if ( filter.accept( file ) ) {
			} else {
				rvalue = false;
				if ( bailOnFirst ) {
					break;
				}
			}
		}
		
		return rvalue;
	}
	
	/**
	 *
	 * Recurse down the directories in the dirname Vector
	 * and pass the files to the StdOutFilter
	 *
	 * @param dirnames	Vector of String names of directories
	 *
	 */
	public final static void find( Vector dirnames ) {
		Vector callbacks = new Vector();
		
		callbacks.addElement( new FileFilter() {
			public void init( String val ) {}
				
			public boolean accept( File f ) {
				return true;
			}

			public String getName() {
				return "YesFileFilter"; 
			}
		} );
		
		find( dirnames, callbacks );
	}
	
	/**
	 *
	 * Recurse down the directories in the dirname Vector
	 * and pass the files to the Vector of callbacks
	 *
	 * @param dirnames	Vector of String names of directories
	 * @param callbacks Vector of FileFilter to invoke
	 *
	 */
	public final static void find( Vector dirnames, Vector callbacks ) {
		Vector filters = new Vector();

		filters.addElement( new FileFilter() {
			public void init( String val ) {}

			public boolean accept( File f ) {
				System.out.println( f );
				return true;
			}

			public String getName() {
				return "StdOutFilter";
			}
		} );

		find( dirnames, callbacks, filters );
	}

	/**
	 *
	 * Recurse down the directories in the dirname Vector,
	 * any which pass all the filters (logical and) and 
	 * sent to each of the callbacks
	 *
	 * @param dirnames	Vector of String names of directories
	 * @param callbacks Vector of FileFilter to invoke
	 * @param filters 	Vector of FileFilter to test each file with
	 *
	 */
	public final static void find( Vector dirnames, Vector callbacks, Vector filters ) {
		
		if ( dirnames.size() == 0 ) dirnames.addElement( "." );

		Stack dirs = new Stack();
		for ( int i = 0 ; i < dirnames.size() ; ++i ) {
			// no need to check as directory, cuz it's check down yonder...
			dirs.push( new File( ( String ) dirnames.elementAt( i ) ) );
		}

		find( dirs, filters, callbacks );
	}

	/**
	 *
	 * Recurse down the directories on the stack, any which pass all the 
	 * filters (logical and) and sent to each of the callbacks
	 *
	 * @param dirs 		Stack of File's to recurse down
	 * @param callbacks Vector of FileFilter to invoke
	 * @param filters 	Vector of FileFilter to test each file with
	 *
	 */
	public final static void find( Stack dirs, Vector callbacks, Vector filters ) {

		while ( !dirs.empty() ) {

			// pop the stack
			File file = ( File ) dirs.pop();

			if ( passesFilters( file, filters ) ) invokeFilters( file, callbacks );

			// someone might pass us some crap
			if ( !file.isDirectory() ) continue;

			File[] files = file.listFiles();
			for ( int i = 0 ; i < files.length ; ++i ) {
				file = files [ i ];
				if ( passesFilters( file, filters ) ) invokeFilters( file, callbacks );

				if ( file.isDirectory() ) dirs.push( file );
			}
		}
	}

};

/**
 *
 * $Log: Find.java,v $
 * Revision 1.5  2005/03/19 17:50:02  brian
 * repackaging
 *
 * Revision 1.4  2004/09/01 01:10:42  brian
 * fix class level javadoc placement
 *
 * Revision 1.3  2004/09/01 00:11:06  brian
 * author, log and header stuff
 *
 *
 */
